<?php

namespace Drupal\simplenews_stats;

use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\Core\Entity\EntityInterface;
use Drupal\simplenews\SubscriberInterface;
use Drupal\simplenews_stats\SimplenewsStatsAllowedLinks;

/**
 * Class SimplenewsStatsMail.
 *
 * @author Damien
 */
class SimplenewsStatsMail {

  protected $message;

  /**
   * @var Drupal\simplenews\Mail\MailEntity 
   */
  protected $SimpleNewsMail;

  /**
   * @var Drupal\simplenews_stats\SimplenewsStatsEngine
   */
  protected $simplenewsStatsEngine;

  /**
   * @var Drupal\simplenews_stats\SimplenewsStatsAllowedLinks 
   */
  protected $simplenewsStatsAllowedLinks;

  public function __construct(SimplenewsStatsEngine $SimplenewsStatsEngine, SimplenewsStatsAllowedLinks $SimplenewsStatsAllowedLinks) {
    $this->simplenewsStatsEngine       = $SimplenewsStatsEngine;
    $this->simplenewsStatsAllowedLinks = $SimplenewsStatsAllowedLinks;
  }

  /**
   * Prepare the mail by adding to it tags and image Tracker.
   */
  public function prepareMail(&$message) {
    $this->message = &$message;

    if (!empty($message['params']['simplenews_mail']) && is_object($message['params']['simplenews_mail']))
      $this->SimpleNewsMail = $message['params']['simplenews_mail'];

    $this->addImageTracker()
        ->addTags()
        ->logHitSent();
  }

  /**
   * Get the body.
   * 
   * @return Drupal\Core\Render\Markup
   */
  protected function getBody() {
    return reset($this->message['body']);
  }

  /**
   * Return the context (Simplenews source object).
   * 
   * @return SimplenewsMail
   */
  protected function getContext() {
    return $this->SimpleNewsMail;
  }

  /**
   * Return the context (Simplenews source object).
   * 
   * @return Drupal\simplenews\SubscriberInterface
   */
  protected function getSubscriber() {
    $SimpleNewsMail = $this->getContext();
    return $SimpleNewsMail->getSubscriber();
  }

  /**
   * Get a tag.
   * 
   * @param Drupal\simplenews\SubscriberInterface $subscriber
   * @param Drupal\Core\Entity\EntityInterface $entity
   * @return string The tag.
   */
  protected function getTag(SubscriberInterface $subscriber, EntityInterface $entity) {
    return 'u' . $subscriber->id() . 'nl' . $entity->id();
  }

  /**
   * Return the entityfrom simplenws object.
   * 
   * @return EntityInterface
   */
  protected function getEntity() {
    $SimpleNewsMail = $this->getContext();
    return $SimpleNewsMail->getEntity();
  }

  /**
   * Add Body.
   * 
   * @param string $string
   */
  protected function setBody($string) {
    $this->message['body'] = [Markup::create($string)];
  }

  /**
   * Adds image tracker to the body.
   */
  protected function addImageTracker() {
    $SimpleNewsSource = $this->getContext();
    $subscriber       = $SimpleNewsSource->getSubscriber();
    $entity           = $SimpleNewsSource->getEntity();

    // Dont add image if this user is not registred.
    if (!$subscriber->id()) {
      return;
    }

    $tag = $this->getTag($subscriber, $entity);
    $Url = Url::fromRoute('simplenews_stats.hit_view')
        ->setOption('query', ['sstc' => $tag])
        ->setAbsolute();

    $image = array(
      '#theme'      => 'image',
      '#attributes' => [
        'src' => $Url->toString()
      ]
    );

    $this->setBody($this->getBody() . render($image));
    return $this;
  }

  /**
   * AddTags on every link in the mail.
   */
  protected function addTags() {
    $body = $this->getBody();

    // Add tags on links.
    $body = preg_replace_callback("`<a.*href=\"([a-zA-Z0-9@:%_+*~#?&=.,/;-]*[a-zA-Z0-9@:%_+*~#&?=/;-])\"`i", [$this, 'replaceLinksUrl'], $body);

    $this->setBody($body);
    return $this;
  }

  /**
   * Callback of AddTags. 
   * 
   * @param string $url
   * @return string
   */
  protected function replaceLinksUrl($url) {
    $SimpleNewsSource = $this->getContext();
    $subscriber       = $SimpleNewsSource->getSubscriber();
    $entity           = $SimpleNewsSource->getEntity();

    $external    = false;
    $url_cleaned = strtolower($url[1]);

    // Escape if email is not registered.
    if (!$subscriber->id())
      return $url[0];

    // Find Url Type.
    if (preg_match('/^https?\:\/\//', $url_cleaned)) {
      $external = true;
      $Url      = Url::fromUri($url[1]);

      if (!$this->simplenewsStatsAllowedLinks->isLinkExist($entity, $url[1])) {
        $this->simplenewsStatsAllowedLinks->add($entity, $url[1]);
      }
      $Url = $this->generateExternalLink($subscriber, $entity, $url[1]);
    }
    else if (substr($url[1], 0, 1) == '/') {
      $Url = Url::fromUri('internal:' . $url[1]);
    }
    else
      $Url = Url::fromUri('internal:/' . $url[1]);

    $tag = $this->getTag($subscriber, $entity);
    if (!$external)
      $Url->setOption('query', ['sstc' => $tag]);
    $Url->setAbsolute();

    return str_replace($url[1], $Url->toString(), $url[0]);
  }

  /**
   * Return a link for external link reference.
   * 
   * @param Drupal\simplenews\SubscriberInterface $subscriber
   * @param Drupal\Core\Entity\EntityInterface $entity
   * @param string $link
   * @return string The link.
   */
  protected function generateExternalLink(SubscriberInterface $subscriber, EntityInterface $entity, $link) {
    $url = Url::fromRoute('simplenews_stats.hit_click', ['tag' => $this->getTag($subscriber, $entity), 'link' => $link]);
    return $url;
  }

  /**
   * Log sent Hit.
   */
  public function logHitSent() {
    $SimpleNewsSource = $this->getContext();
    $entity           = $SimpleNewsSource->getEntity();

    $this->simplenewsStatsEngine->logHitSent($this->getSubscriber(), $entity);
  }

}
